@@ -89,8 +89,7 @@ class Gencontrol(Base):
     ) -> None:
         super().do_main_setup(config, vars, makeflags)
         makeflags.update({
-            'VERSION': self.version.linux_version,
-            'UPSTREAMVERSION': self.version.linux_upstream_full,
+            'UPSTREAMVERSION': self.version.linux_version,
             'ABINAME': self.abiname,
             'SOURCEVERSION': self.version.complete,
         })
@@ -437,8 +436,11 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
                 self.bundle.add('image-extra-dev', ruleid, makeflags, vars, arch=arch)
             )
 
-        # In a quick build, only build the quick flavour (if any).
-        if not config.defs_flavour.is_quick:
+        # In a quick build, only build the test flavour.
+        if config.defs_flavour.is_test:
+            for package in packages_own:
+                package.build_profiles[0].pos.add('pkg.linux.quick')
+        else:
             for package in packages_own:
                 package.build_profiles[0].neg.add('pkg.linux.quick')
 
@@ -461,7 +463,7 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
         self.tests_control.extend(tests_control_headers)
 
         kconfig = []
-        for c in config.config:
+        for c in (config.config_nodefault if config.defs_flavour.is_test else config.config):
             for d in self.config_dirs:
                 if (f := d / c).exists():
                     kconfig.append(str(f))
@@ -596,14 +598,15 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
 
     def write_signed(self) -> None:
         for bundle in self.bundles.values():
-            pkg_sign_entries = {}
+            pkg_sign_entries_notquick = {}
+            pkg_sign_entries_quick = {}
 
             for p in bundle.packages.values():
                 if not isinstance(p, BinaryPackage):
                     continue
 
                 if pkg_sign_pkg := p.meta_sign_package:
-                    pkg_sign_entries[pkg_sign_pkg] = {
+                    e = {
                         'trusted_certs': [],
                         'files': [
                             {
@@ -614,9 +617,16 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
                         ],
                     }
 
-            if pkg_sign_entries:
-                with bundle.path('files.json').open('w') as f:
-                    json.dump({'packages': pkg_sign_entries}, f, indent=2)
+                    if 'pkg.linux.quick' in p.build_profiles[0].pos:
+                        pkg_sign_entries_quick[pkg_sign_pkg] = e
+                    else:
+                        pkg_sign_entries_notquick[pkg_sign_pkg] = e
+
+            if pkg_sign_entries_notquick or pkg_sign_entries_quick:
+                with bundle.path('files.notquick.json').open('w') as f:
+                    json.dump({'packages': pkg_sign_entries_notquick}, f, indent=2)
+                with bundle.path('files.quick.json').open('w') as f:
+                    json.dump({'packages': pkg_sign_entries_quick}, f, indent=2)
 
     def write_tests_control(self) -> None:
         with open("debian/tests/control", 'w') as f:
